import java.io.*;

class RenameScreen
{
	// We want to rename every file that starts with ScreenSelectMaster to start with SSMaster
	public static void renameFiles(File path, String prefix, String toPrefix)
	{
		File [] files = path.listFiles();
		
		for(int i = 0; i < files.length; i++)
		{
			String oldFilename = files[i].getName();
			String filename = files[i].getName();
			
			if(filename.toLowerCase().startsWith(prefix.toLowerCase()))
			{
				if(filename.length() == prefix.length())
					filename = toPrefix;
				else
				{
					filename = filename.substring(prefix.length());
					filename = toPrefix + filename;
				}
				
				File newFile = new File(path, filename);
				
				files[i].renameTo(newFile);
				System.out.println(oldFilename + " -> " + filename);
			}
			
			if(files[i].isDirectory())
				renameFiles(files[i], prefix, toPrefix);
		}
	}
	
	public static void replaceStrings(File path, String prefix, String toPrefix) throws IOException
	{
		File [] files = path.listFiles();
		
		for(int i = 0; i < files.length; i++)
		{
			if(files[i].isFile() && files[i].getName().toLowerCase().endsWith(".redir"))
			{
				BufferedReader buf = new BufferedReader(new FileReader(files[i]));
				
				String line = buf.readLine();
				if(line.indexOf(prefix) != -1)
				{
					System.out.println("Replacing strings in " + files[i].getName());
					String newLine = line.replaceAll("\\Q" + prefix + "\\E", toPrefix);
					buf.close();
					
					PrintWriter out = new PrintWriter(new FileWriter(files[i], false));
					out.println(newLine);
					out.close();
				}
				else
					buf.close();
			}
			else if(files[i].isFile() && files[i].getName().toLowerCase().endsWith(".ini"))
			{
				BufferedReader buf = new BufferedReader(new FileReader(files[i]));
				boolean found = false;
				
				while(buf.ready())
				{
					String line = buf.readLine();
					if(line.indexOf(prefix) != -1)
					{
						System.out.println("Replacing strings in " + files[i].getName());
						found = true;
						break;
					}
				}
				buf.close();
				
				if(found)
				{
					File tempFile = new File(path, "temp~~~.~~~");
					tempFile.delete();
					BufferedReader in = new BufferedReader(new FileReader(files[i]));
					PrintWriter out = new PrintWriter(new FileWriter(tempFile), false);
					
					while(in.ready())
					{
						String line = in.readLine();
						if(line.indexOf(prefix) != -1)
						{
							line = line.replaceAll("\\Q" + prefix + "\\E", toPrefix);
						}
						out.println(line);
					}
					in.close();
					out.close();
					files[i].delete();
					tempFile.renameTo(files[i]);
				}
			}
			if(files[i].isDirectory())
				replaceStrings(files[i], prefix, toPrefix);
		}
	}
	
	public static void main(String [] args) throws IOException
	{
		if(args.length < 3)
		{
			System.out.println("usage java RenameScreen <path> <from> <to>");
			return;
		}
		
		System.out.println("Renaming files...");
		
		File path = new File(args[0]);
		if(!path.isDirectory())
		{
			System.out.println("Path not found...");
			return;
		}
		
		renameFiles(path, args[1], args[2]);
		
		System.out.println("Searching and replacing strings in files");
		
		replaceStrings(path, args[1], args[2]);
	}
}

/*
 * (c) 2004 Ryan Dortmans
 * All rights reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, and/or sell copies of the Software, and to permit persons to
 * whom the Software is furnished to do so, provided that the above
 * copyright notice(s) and this permission notice appear in all copies of
 * the Software and that both the above copyright notice(s) and this
 * permission notice appear in supporting documentation.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF
 * THIRD PARTY RIGHTS. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR HOLDERS
 * INCLUDED IN THIS NOTICE BE LIABLE FOR ANY CLAIM, OR ANY SPECIAL INDIRECT
 * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
 * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */